#
# Generate the code for a bioperl module
#

package Maker;

my $command=$ARGV[0];

my $perldef = "/local/gensoft/lib/Pise/5.a/PerlDef/$command.pm";
require $perldef;

my $defs = $command->new;
my $title = "Bioperl class for:\n\n\t";
$title .= $defs->title . "\t" . $defs->description;
if ($defs->authors) {
    $title .= " (" . $defs->authors . ")";
}
if (defined $defs->reference) {
    $title .= "\n\n\tReferences:\n";
    foreach my $ref ($defs->reference) {
	$title .= "\n\t\t" . $ref . "\n"; 
    }
}

print "# \$Id\$
# BioPerl module for Bio::Tools::Run::PiseApplication::$command
#
# Cared for by Catherine Letondal <letondal\@pasteur.fr>
#
# For copyright and disclaimer see below.
#
# POD documentation - main docs before the code

=head1 NAME

Bio::Tools::Run::PiseApplication::$command

=head1 SYNOPSIS

  #

=head1 DESCRIPTION

Bio::Tools::Run::PiseApplication::$command

      $title


      Parameters: 

        (see also:
          http://bioweb.pasteur.fr/seqanal/interfaces/$command.html 
         for available values):

";

foreach my $param ($defs->parameters_order) {
    my $type = $defs->type($param);
    my $prompt = $defs->prompt($param);
    my %pipein = $defs->pipein($param);
    my %pipeout = $defs->pipeout($param);

    next if ($type eq "Results");

    next if ($type eq "Paragraph");

    print "\n\t\t$param ($type)\n";
    if (defined $prompt && $prompt ne "") {
	print "\t\t\t$prompt\n";
    }
    
    foreach my $in (keys %pipein) {
	if ($in) {
	    print "\t\t\tpipe: $in\n";
	}
    }
    foreach my $out (keys %pipeout) {
	if ($out) {
	    print "\t\t\tpipe: ",$pipeout{$out},"\n";
	}
    }
}


print "
=head1 FEEDBACK

=head2 Mailing Lists

User feedback is an integral part of the evolution of this and other
Bioperl modules. Send your comments and suggestions preferably to
the Bioperl mailing list.  Your participation is much appreciated.

  bioperl-l\@bioperl.org              - General discussion
  http://bioperl.org/MailList.shtml  - About the mailing lists

=head2 Reporting Bugs

Report bugs to the Bioperl bug tracking system to help us keep track
of the bugs and their resolution. Bug reports can be submitted via
email or the web:

  bioperl-bugs\@bioperl.org
  http://bioperl.org/bioperl-bugs/

=head1 AUTHOR

Catherine Letondal (letondal\@pasteur.fr)

=head1 COPYRIGHT

Copyright (C) 2003 Institut Pasteur \& Catherine Letondal.
All Rights Reserved.

This module is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=head1 DISCLAIMER

This software is provided \"as is\" without warranty of any kind.

=head1 SEE ALSO

=over

=item *

http://bioweb.pasteur.fr/seqanal/interfaces/$command.html

=item *

Bio::Tools::Run::PiseApplication

=item *

Bio::Tools::Run::AnalysisFactory::Pise

=item *

Bio::Tools::Run::PiseJob

=back

=cut

#'
package Bio::Tools::Run::PiseApplication::$command;

use vars qw(\@ISA);
use strict;
use Bio::Tools::Run::PiseApplication;

\@ISA = qw(Bio::Tools::Run::PiseApplication);

=head2 new

 Title   : new()
 Usage   : my \$$command = Bio::Tools::Run::PiseApplication::$command->new(\$location, \$email, \@params);
 Function: Creates a Bio::Tools::Run::PiseApplication::$command object.
           This method should not be used directly, but rather by 
           a Bio::Tools::Run::AnalysisFactory::Pise instance.
           my \$factory = Bio::Tools::Run::AnalysisFactory::Pise->new();
           my \$$command = \$factory->program('$command');
 Example : -
 Returns : An instance of Bio::Tools::Run::PiseApplication::$command.

=cut

sub new {
    my (\$class, \$location, \$email, \@params) = \@_;
    my \$self = \$class->SUPER::new(\$location, \$email);

";

print "# -- begin of definitions extracted from $perldef\n\n";
open(DEF,$perldef) || die "cannot open $perldef: $!";
my $print = -1;
my $param_list = 0;
my $param;
my $line;
my $prompt;
while (<DEF>) {
    $print = 1 if (/\{COMMAND\}/ && $print == -1);
    $print = 0 if (/bless/);
    $param_list++ if ($param_list == 1);
    $param_list = 1 if (/PARAMETERS_ORDER/);
    $param_list = 3 if ($param_list == 2 && /;/);
    
    if ($param_list == 2) {
	$line = $_;
	$param = $line;
	chomp($param);
	$param =~ s/\"(.+)\",/$1/;
	$param =~ s/\s+//g;
	$prompt = $defs->prompt($param);
	#print STDERR "$param_list param: $param ($prompt)\n";
	if ($prompt) {
	    $line =~ s/\",/\", \t\# $prompt/;
	}
	print $line if $print == 1;
    } else {
	print if $print == 1;
    }
    #print STDERR "$print $_";
}
close DEF;
print "# -- end of definitions extracted from $perldef\n\n";

print "

    \$self->_init_params(\@params);

    return \$self;
}



1; # Needed to keep compiler happy

";

