# 
# utilities for the tcl interface generator
# 
# TODO:
# - disable delete job (double-2) on non finished jobs
# - iconify form
# - mark unfinished jobs in the list
# - size of the output windows/frames
# - save, connect, ... button in output windows/frame
# - pouvoir donner des noms aux requetes
# - desactiver submit, afficher les jobs pas finis

global pise_defs
if [info exists pise_defs] {
    return
}
set pise_defs 1

global pise_jobs_file
global env
set pise_jobs_file "$env(HOME)/.pise"

proc post_request {command multipart} {

    #puts "post_request"

    global env
    if [info exists env(PISE_SERVER)] {
	set pise_server $env(PISE_SERVER)
    } else {
	set pise_server "http://bioweb.pasteur.fr/cgi-bin/seqanal"
    }
    puts stderr "(post_request) pise_server: $pise_server"

    #set token [ http::geturl http://bioweb.pasteur.fr/cgi-bin/seqanal/$command.pl -multipart $multipart ]
    set token [http::geturl ${pise_server}/$command.pl -multipart $multipart]

    set status [http::status $token]
    if {$status == "ok"} {
        set code [http::code $token]
        if [regexp {200} $code match] {
            set results [http::data $token]
	    set tmp [open /tmp/w w]
	    puts $tmp $results
	    close $tmp
	    #puts "post_request: results=\n$results"
	    set found [xmlparse $command [clean_xhtml $results]]
        }  elseif [regexp {404} $code match] {
	    puts stderr "post_request($command): $code"
        } else {
           puts stderr "post_request($command): $code"
        }
    } else {
        puts stderr "post_request($command): $status"
    }

}

proc xmlparse {command results {index ""} } {
    global xmlparser
    global job_terminated
    global hrefs
    global connected
    global scratch_dir
    global jobid

    set outputs_frame [get_outputs_frame $command]

    if { ! [info exists xmlparser] } {
	set xmlparser [xml::parser dexml]
	$xmlparser configure -elementstartcommand elementstart -elementendcommand elementend -characterdatacommand pcdata
    }
    $xmlparser parse $results

    global env
    if [info exists env(PISE_RESULTS)] {
	set pise_results $env(PISE_RESULTS)
    } else {
	set pise_results "http://bioweb.pasteur.fr/seqanal/tmp"
    }
    puts stderr "(xmlparse) pise_results: $pise_results"

    #set jobid "http://bioweb.pasteur.fr/seqanal/tmp/"
    set jobid "$pise_results/"
    append jobid $scratch_dir
    append jobid "/"
    if {! $job_terminated } {
	fill_jobs_list $command $jobid 0   
    } else {
	fill_jobs_list $command $jobid 1
	foreach {url} $hrefs {
	    print_url $command $outputs_frame $url
	}
    }
    return $job_terminated
}

proc elementstart {name attlist} {
    global job_terminated
    global hrefs
    global last_href
    global connected
    global scratch_dir

    if {$name == "HTML"} {
	set job_terminated 1
    } elseif {$name == "A"} {
	foreach {att_name att_value} $attlist {
	    if {$att_name == "HREF"} {
		set mailto ""
		set pise ""
		regexp {Pise} $att_value pise
		regexp {mailto:} $att_value mailto
		if {$mailto == "" && $pise == ""} {
		    set last_href $att_value
		    lappend hrefs $att_value
		}
	    }
	}
    } elseif {$name == "FORM"} {
	foreach {att_name att_value} $attlist {
	    if {$att_name == "action"} {
		if [regexp {connect.pl} $att_value match] {
		    set connected($last_href) 1
		} elseif  [regexp {results.pl} $att_value match] {
		    set job_terminated 0
		}
	    }
	}
    } elseif {$name == "INPUT"} {
	set name [lookup_attvalue "NAME" $attlist]
	set value [lookup_attvalue "VALUE" $attlist]
	if {$name == "scratch_dir"} {
	    set scratch_dir $value
	}
    } elseif {$name == "OPTION"} {
	if {[info exists last_href] && $last_href != "" } {
	    if {[array exists connected] && [info exists connected($last_href)] && $connected($last_href)} {
		# array of list in tcl?
	    }
	}
    }
}

proc elementend {name} {
    global last_href
    if {$name == "SELECT"} {
	set last_href ""
    }
}

proc pcdata data {
    global job_terminated
    if [regexp {Results not available yet} $data match] {
	set job_terminated 0
    }
}

proc lookup_attvalue {attname attlist} {
    foreach {name value} $attlist {
	if {$name == $attname} {
	    return $value
	}
    }
}

proc clean_xhtml text {
    # some parsers want &amp; in attribute list (Perl/SAX)
    # other only in text (TclXML)
    regsub -all {&gt;} $text {GT} text
    regsub -all {&lt;} $text {LT} text
    regsub -all {&amp;} $text {AMP} text
    return $text
}

proc look4files {command results {index ""} } {

    global env
    if [info exists env(PISE_RESULTS)] {
	set pise_results $env(PISE_RESULTS)
    } else {
	set pise_results "http://bioweb.pasteur.fr/seqanal/tmp"
    }
    puts stderr "(look4files) pise_results: $pise_results"

    set outputs_frame [get_outputs_frame $command]
    set output 0
    foreach {line} [split $results \n] {
      if [regexp {upon completion .* <a href="(.*)" TARGET="_top">} $line href url] { 
         fill_jobs_list $command $url 0
         set output 1
      } elseif [regexp {<a href="(.*)" TARGET="_top">} $line href url] {
	  
	  #regexp {(http://bioweb.pasteur.fr/seqanal/tmp/.+/).*} $url match index 
	  regexp "(${pise_results}/.+/).*" $url match index 
         append index "index.html"
         fill_jobs_list $command $index
         print_url $command $outputs_frame $url
         set output 1


      } elseif [regexp "${pise_results}/(.*)\"" $line href subdir] {
#  (if) [regexp {http://bioweb.pasteur.fr/seqanal/tmp/(.*)\"} $line href subdir] 
#         regsub {\"} http://bioweb.pasteur.fr/seqanal/tmp/$subdir {} url
#         regexp {(http://bioweb.pasteur.fr/seqanal/tmp/.+/).*} $url match index 
         regsub {\"} ${pise_results}/$subdir {} url
         regexp "(${pise_results}/.+/).*" $url match index 
         append index "index.html"
         fill_jobs_list $command $index
         print_url $command $outputs_frame $url
         set output 1
      }
    }
    if { $output == 0 } {
         print_text $command $results $index
    }

    return $output

}

proc look4job {command url} {
    set token [http::geturl $url]
    set status [http::status $token]
    if {$status == "ok"} {
        set code [http::code $token]
        if [regexp {200} $code match] {
            set results [http::data $token]
	    set tmp [open /tmp/w w]
	    puts $tmp $results
	    close $tmp
	    set found [xmlparse $command [clean_xhtml $results]]
        } elseif [regexp {404} $code match] {
           remove_from_jobs_list $command $url
        } else {
           puts stderr "$url: $code"
        }
    } else {
        puts stderr "$url: $status"
    }
}

proc get_outputs_frame {command} {
    global root_frame
    if [info exists root_frame] {
	set frame [string tolower $root_frame\.$command.outputs]
    } else {
	set frame [string tolower \.$command.outputs]
    }
    if { [winfo exists $frame] } {
        foreach {child} [winfo children $frame] {
           if [winfo exists $child] {
              destroy $child
           }
        }
        return $frame
    } else {
	frame $frame -relief ridge -bd 2
	set row [current_row $command]
	grid $frame -row $row -column 0 -columnspan 2
	set outputs($command) $frame
	return $outputs($command) 
    }

}

proc print_url {command outputs_frame url} {

    set path_parts [split $url '/']
    set file [lindex $path_parts [expr [llength $path_parts] - 1]]
    regsub -all {\.} $file {_} frame
    set frame [string tolower $frame]

    if [file exists $file] {
        return;
    }

    set token [http::geturl $url]
    set status [http::status $token]
    if { $status == "ok" } {
	set code [http::code $token]
	if [regexp {200} $code match] {
	    set data [http::data $token]
	    file_button $command $outputs_frame.$frame $file $data $url
	}
    }
}


proc print_text {command text url} {
    set outputs_frame [get_outputs_frame $command]
    set frame [string tolower "$outputs_frame.$command"]
    append frame "_output"
    if [winfo exists $frame] {
	foreach {child} [winfo children $frame] {
	    destroy $child
	}
    } else {
	frame $frame
	pack $frame
    }
    text_frame $command $frame "$command output" $text $url
}

proc file_button {command frame file data url} {
    if [winfo exists $frame] {
	foreach {child} [winfo children $frame] {
	    destroy $child
	}
    } else {
	frame $frame
    }
    frame $frame.h
    pack $frame -side top -fill both -expand true
    pack $frame.h -side top -fill both -expand true
    
    label $frame.h.label  -text "$file ($url)" 
    button $frame.h.b -text "Open" -command [list text_frame $command $frame $file $data $url 1]
    pack $frame.h.label -side left
    pack $frame.h.b -side left
}

proc text_frame {command frame file data {url ""} {top 0}} {
     if [winfo exists $frame] {
	 foreach {child} [winfo children $frame] {
	     destroy $child
	 }
     } else {
	 frame $frame
	 pack $frame -side top -fill both -expand true
     }

     frame $frame.h
     pack $frame.h -side top -fill both -expand true

     label $frame.h.label  -text "$file ($url)" 
     pack $frame.h.label -side left

     set width 80

     set parent [winfo parent $frame]
     regsub $parent $frame {} orphan 
     if [winfo exists $orphan] {
	 destroy $orphan
     }
     toplevel $orphan
     frame $orphan.b
     pack $orphan.b -side top -fill both -expand true
     set body $orphan.b
     button $orphan.close -text "Close $url" \
	     -command [list close_output_frame $command $frame $orphan $file $data $url 1]
     pack $orphan.close -side top

     button $frame.h.b -text "Close" -command [list close_output_frame $command $frame $orphan $file $data $url 1]
     pack $frame.h.b -side left

     set height [llength [split $data \n]]
     if {$height > 40} {
	 set height 40
     }

     set output $body.output
     set scrollbar $body.scroll
     text $output -width $width -height $height \
	-borderwidth 2 -relief raised -setgrid true \
	-yscrollcommand [list $scrollbar set] 
     scrollbar $scrollbar -command [list $output yview]

     pack $scrollbar -side right -fill y
     pack $output -side left -fill both -expand true

     foreach {line} [split $data \n] {
	$output insert end $line\n
     }

     return $output
}

proc close_output_frame {command frame body file data url {top 0}} {
     destroy $body
     $frame.h.b configure -text "ReOpen" -command [list text_frame $command $frame $file $data $url $top]
}

proc jobs_frame {command} {
    #puts "jobs_frame"
    global root_frame
    if [info exists root_frame] {
	return [string tolower $root_frame\.$command.jobs]
    } else {
	return [string tolower \.$command.jobs]
    }
}

proc init_jobs_list {command} {

    #puts "init_jobs_list command: $command"
    global pise_jobs_file
    #puts "init_jobs_list: pise_jobs_file=$pise_jobs_file"
    set jobs [jobs_frame $command]
    if [file exists $pise_jobs_file] {
       set previous_jobs [open $pise_jobs_file]
       foreach {line} [split [read $previous_jobs] \n] {
          if { $line != "" && [regexp "$command/" $line match] } {
              fill_jobs_list $command $line
          }
       }
       close $previous_jobs
    }
} 

proc save_jobs {command} {
    #puts "save_jobs"
    global pise_jobs_file
    if [file exists $pise_jobs_file] {
	set previous_jobs [open $pise_jobs_file]
	set i 0
	foreach {line} [split [read $previous_jobs] \n] {
	    if { $line != "" && ! [regexp "$command/" $line match]  } {
		if { ! [info exists other_jobs] } {
		    set other_jobs [list $line]
		} else {
		    set other_jobs [linsert $other_jobs $i $line]
		}
		incr i
	    }
	}
	close $previous_jobs
    }
    set jobs [jobs_frame $command]
    set urls [$jobs.list get 0 end]
    set saved_jobs [open $pise_jobs_file w]
    if [info exists other_jobs] {
	foreach u $other_jobs {
	    puts $saved_jobs $u
	}
    }
    foreach {u} $urls {
	
	regsub {\* } $u {} u
	puts $saved_jobs $u
    }
    close $saved_jobs
}

proc fill_jobs_list {command url {finish 1}} {

    #puts "fill_jobs_list"
    set jobs [jobs_frame $command]
    set i [search_jobs_list $command $url]
    if { $i < 0} {
	if {$finish == 0} {
	    $jobs.list insert end "* $url"
	} else {
	    $jobs.list insert end $url
	}
    } elseif {$finish == 1} {
	set u [$jobs.list get $i]
	if [regexp {\* } $u match] {
	    regsub {\* } $u {} new_url
	    $jobs.list delete $i
	    $jobs.list insert $i $new_url
	}
    }
    #puts "end of fill_jobs_list"

} 

proc remove_from_jobs_list {command url} {
    #puts "remove_from_jobs_list"
    set jobs [jobs_frame $command]
    set urls [$jobs.list get 0 end]
    for {set x 0} {$x < [llength $urls]} {incr x} {
       if { [lindex $urls $x] == $url} {
           $jobs.list delete $x
       }
    }
    return -1
}

proc search_jobs_list {command url} {
    #puts "search_jobs_list"
    set jobs [jobs_frame $command]
    set urls [$jobs.list get 0 end]
    for {set x 0} {$x < [llength $urls]} {incr x} {
	set u [lindex $urls $x]
	regsub {\* } $u {} u
	if { $u == $url} {
	    return $x
	}
    }
    return -1
}

proc QuitCommand {command} {
    save_jobs $command
    global root_frame
    if {[info exists root_frame] && $root_frame != ""} {
	destroy [string tolower $root_frame.$command]
    } else {
	destroy [string tolower .$command]
    }
    if {[info exists root_frame] && $root_frame != ""} {
	set childs [winfo children $root_frame]
    } else {
	set childs [winfo children .]
    }
    if [info exists childs] {
	if {[llength $childs] == 0} {
	    if {[info exists root_frame] & $root_frame != ""} {
		destroy $root_frame
	    } else {
		exit
	    }
	} elseif { [llength $childs] == 1 && [lindex $childs 0] == ".__tk_filedialog"} {
	    if {[info exists root_frame] & $root_frame != ""} {
		destroy $root_frame
	    } else {
		exit
	    }
	}
    } else {
	if {[info exists root_frame] & $root_frame != ""} {
	    destroy $root_frame
	} else {
	    exit
	}
    }
}

proc QuitPise {} {
    # foreach command
    #save_jobs $command
    #destroy .$command

    exit
}

proc getFile {prompt param} {
    upvar $param x
    global custom_getOpenFile
    set p [info procs custom_getOpenFile]
    if {[info procs custom_getOpenFile] != ""} {
	puts stderr "there is a custom getOpenFile..."
	set x [custom_getOpenFile $x $prompt]
    } else {
	set x [tk_getOpenFile -title $prompt]
    }
}

proc current_row {command} {
    global current_row
    #puts "pise.tcl (sent code) current_row"
    if {[array exists current_row] && [info exists current_row($command)]} {
        incr current_row($command)
    } else {
        set current_row($command) 0
    }
    return $current_row($command)
}

proc print_quit_submit {command} {
    
    #puts "pise.tcl (sent code) print_quit_submit"
    global root_frame
    if [info exists root_frame] {
	set quit [string tolower $root_frame.$command.quit]
	set submit [string tolower $root_frame.$command.submit]
    } else {
	set quit [string tolower .$command.quit]
	set submit [string tolower .$command.submit]
    }
    set row [current_row $command]
    button $quit -text "Close $command" -command [list QuitCommand $command]
    grid $quit -row $row -column 0

    button $submit -text "Submit $command" \
	    -command [list submit_command_$command $submit]
    grid $submit -row $row -column 1
}

proc select_job {command w} {
    set job [$w get [$w curselection]]
    regsub {\* } $job {} job
    look4job $command $job
}

proc print_jobs_frame  {command} {
    #puts "print_jobs_frame"
    global root_frame
    if [info exists root_frame] {
	set jobs [string tolower $root_frame.$command.jobs]
    } else {
	set jobs [string tolower .$command.jobs]
    }
    if [winfo exists $jobs] { 
       destroy $jobs
    }
    #puts "print_jobs_frame jobs: $jobs"
    frame $jobs
    set row [current_row $command]
    grid $jobs -row $row -column 0 -columnspan 2
    
    listbox $jobs.list -selectmode single -height 0 -width 100
    pack $jobs.list -side top -fill both
    bind $jobs.list <Double-1> [list select_job $command %W]
    bind $jobs.list <Double-2> {%W delete [%W nearest %y]}
    #puts "calling init_jobs_list"
    init_jobs_list $command

}
